@@ -26,7 +26,8 @@
 #include "token.h"
 
 /* backend conf */
-#define TOK_CFG_MAGICCONF  TOKEN_BACKEND(1)
+#define TOK_CFG_MAGICCONF    TOKEN_BACKEND(1)
+#define TOK_CFG_STATIC_DATA  TOKEN_BACKEND(2)
 
 
 /* ext_level's and ext_parm's tokens */
@@ -38,5 +39,6 @@
 
 extern const struct rig_caps dummy_caps;
 extern const struct rig_caps netrigctl_caps;
+extern const struct rig_caps flrig_caps;
 
 #endif /* _DUMMY_H */
