@@ -32,6 +32,8 @@
 #include <errno.h>
 
 #include "hamlib/rig.h"
+#include "network.h"
+#include "serial.h"
 #include "iofunc.h"
 #include "misc.h"
 #include "num_stdio.h"
@@ -50,6 +52,13 @@ static int netrigctl_transaction(RIG *rig, char *cmd, int len, char *buf)
 {
   int ret;
 
+  /* flush anything in the read buffer before command is sent */
+  if (rig->state.rigport.type.rig == RIG_PORT_NETWORK || rig->state.rigport.type.rig == RIG_PORT_UDP_NETWORK) {
+        network_flush(&rig->state.rigport);
+  } else {
+      serial_flush(&rig->state.rigport);
+  }
+
   ret = write_block(&rig->state.rigport, cmd, len);
   if (ret != RIG_OK)
 	return ret;
@@ -235,6 +244,13 @@ chan_t chan_list[CHANLSTSIZ]; /*!< Channel list, zero ended */
 	return (ret < 0) ? ret : -RIG_EPROTO;
 
   rs->has_get_level = strtol(buf, NULL, 0);
+  if (rs->has_get_level & RIG_LEVEL_RAWSTR)
+    {
+      /* include STRENGTH because the remote rig may be able to
+         provide a front end emulation, if it can't then an
+         -RIG_EINVAL will be returned */
+      rs->has_get_level |= RIG_LEVEL_STRENGTH;
+    }
 
   ret = read_string(&rig->state.rigport, buf, BUF_MAX, "\n", sizeof("\n"));
   if (ret <= 0)
@@ -393,6 +409,7 @@ static int netrigctl_get_vfo(RIG *rig, vfo_t *vfo)
   len = sprintf(cmd, "v\n");
 
   ret = netrigctl_transaction(rig, cmd, len, buf);
+  if (ret == -RIG_ENAVAIL) return ret;
   if (ret <= 0)
 	return (ret < 0) ? ret : -RIG_EPROTO;
 
@@ -1331,7 +1348,7 @@ const struct rig_caps netrigctl_caps = {
   .status =         RIG_STATUS_BETA,
   .rig_type =       RIG_TYPE_OTHER,
   .targetable_vfo = 	 0,
-  .ptt_type =       RIG_PTT_RIG,
+  .ptt_type =       RIG_PTT_RIG_MICDATA,
   .dcd_type =       RIG_DCD_RIG,
   .port_type =      RIG_PORT_NETWORK,
   .timeout = 2000,	/* enough for a network */
