@@ -33,6 +33,7 @@
 #include <ctype.h>
 
 #include "hamlib/rig.h"
+#include "network.h"
 #include "serial.h"
 #include "misc.h"
 #include "register.h"
@@ -242,7 +243,13 @@ int kenwood_transaction(RIG *rig, const char *cmdstr, char *data, size_t datasiz
         len++;
       }
 
-      serial_flush(&rs->rigport);
+      /* flush anything in the read buffer before command is sent */
+      if (rs->rigport.type.rig == RIG_PORT_NETWORK || rs->rigport.type.rig == RIG_PORT_UDP_NETWORK) {
+            network_flush(&rs->rigport);
+      } else {
+          serial_flush(&rs->rigport);
+      }
+
       retval = write_block(&rs->rigport, cmd, len);
 
       free(cmd);
@@ -594,8 +601,11 @@ int kenwood_open(RIG *rig)
       || !strcmp ("ID905", id)  /* PowerSDR Flex-6500 */
       || !strcmp ("ID906", id)  /* PowerSDR Flex-6700R */
       || !strcmp ("ID907", id)  /* PowerSDR Flex-6300 */
+      || !strcmp ("ID908", id)  /* PowerSDR Flex-6400 */
+      || !strcmp ("ID909", id)  /* PowerSDR Flex-6600 */
       )
     {
+      priv->is_emulation = 1;   /* Emulations don't have SAT mode */
       strcpy (id, "ID019");  /* fake it */
     }
 
@@ -729,25 +739,25 @@ int kenwood_set_vfo(RIG *rig, vfo_t vfo)
   }
 
   //if rig=ts2000 then check Satellite mode status
-  if(rig->caps->rig_model == RIG_MODEL_TS2000) {
-	  char retbuf[20];
-	  rig_debug(RIG_DEBUG_VERBOSE, "Checking Satellite mode status\n");
-	  sprintf(cmdbuf, "SA");
+  if(rig->caps->rig_model == RIG_MODEL_TS2000 && !priv->is_emulation) {
+    char retbuf[20];
+    rig_debug(RIG_DEBUG_VERBOSE, "Checking Satellite mode status\n");
+    snprintf(cmdbuf, sizeof (cmdbuf), "SA");
 
-	  retval = kenwood_transaction(rig, cmdbuf, retbuf, 20);
-	  if (retval != RIG_OK)
-	    return retval;
+    retval = kenwood_transaction(rig, cmdbuf, retbuf, 20);
+    if (retval != RIG_OK)
+      return retval;
 
-	  rig_debug(RIG_DEBUG_VERBOSE, "Satellite mode status %s\n",retbuf);
-	  //Satellite mode ON
-	  if(retbuf[2]=='1') {
-		  //SAT mode doesn't allow FR command (cannot select VFO)
-		  //selecting VFO is useless in SAT MODE
-		  return RIG_OK;
-		  }
-	  }
+    rig_debug(RIG_DEBUG_VERBOSE, "Satellite mode status %s\n",retbuf);
+    //Satellite mode ON
+    if(retbuf[2]=='1') {
+      //SAT mode doesn't allow FR command (cannot select VFO)
+      //selecting VFO is useless in SAT MODE
+      return RIG_OK;
+      }
+    }
 
-  sprintf(cmdbuf, "FR%c", vfo_function);
+  snprintf(cmdbuf, sizeof (cmdbuf), "FR%c", vfo_function);
 
   if (rig->caps->rig_model == RIG_MODEL_TS50
       || rig->caps->rig_model == RIG_MODEL_TS940)
@@ -803,7 +813,7 @@ int kenwood_set_vfo_main_sub(RIG *rig, vfo_t vfo)
     return -RIG_EINVAL;
   }
 
-  sprintf(cmdbuf, "CB%c", vfo_function);
+  snprintf(cmdbuf, sizeof (cmdbuf), "CB%c", vfo_function);
   return kenwood_transaction(rig, cmdbuf, NULL, 0);
 }
 
@@ -854,7 +864,7 @@ int kenwood_set_split_vfo(RIG *rig, vfo_t vfo, split_t split, vfo_t txvfo)
           retval = kenwood_set_vfo_main_sub (rig, RIG_VFO_MAIN);
           if (retval != RIG_OK) return retval;
         }
-      sprintf(cmdbuf, "TB%c", RIG_SPLIT_ON == split ? '1' : '0');
+      snprintf(cmdbuf, sizeof (cmdbuf), "TB%c", RIG_SPLIT_ON == split ? '1' : '0');
       return kenwood_transaction(rig, cmdbuf, NULL, 0);
     }
 
@@ -868,7 +878,7 @@ int kenwood_set_split_vfo(RIG *rig, vfo_t vfo, split_t split, vfo_t txvfo)
       return -RIG_EINVAL;
     }
     /* set RX VFO */
-    sprintf(cmdbuf, "FR%c", vfo_function);
+    snprintf(cmdbuf, sizeof (cmdbuf), "FR%c", vfo_function);
     retval = kenwood_transaction(rig, cmdbuf, NULL, 0);
     if (retval != RIG_OK)
       return retval;
@@ -903,7 +913,7 @@ int kenwood_set_split_vfo(RIG *rig, vfo_t vfo, split_t split, vfo_t txvfo)
       }
   }
   /* set TX VFO */
-  sprintf(cmdbuf, "FT%c", vfo_function);
+  snprintf(cmdbuf, sizeof (cmdbuf), "FT%c", vfo_function);
   retval = kenwood_transaction(rig, cmdbuf, NULL, 0);
   if (retval != RIG_OK)
     return retval;
@@ -930,7 +940,7 @@ int kenwood_set_split(RIG *rig, vfo_t vfo, split_t split, vfo_t txvfo)
   char cmdbuf[6];
   int retval;
 
-  sprintf(cmdbuf, "SP%c", RIG_SPLIT_ON == split ? '1' : '0');
+  snprintf(cmdbuf, sizeof (cmdbuf), "SP%c", RIG_SPLIT_ON == split ? '1' : '0');
   retval = kenwood_transaction(rig, cmdbuf, NULL, 0);
   if (retval != RIG_OK)
     return retval;
@@ -1087,11 +1097,18 @@ int kenwood_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
     return -RIG_EINVAL;
 
   char freqbuf[16];
-  unsigned char vfo_letter;
+  unsigned char vfo_letter = '\0';
   vfo_t tvfo;
+  int err;
 
   tvfo = (vfo==RIG_VFO_CURR || vfo==RIG_VFO_VFO) ? rig->state.current_vfo : vfo;
 
+  if (RIG_VFO_CURR == tvfo) {
+    /* fetch from rig */
+    err = rig_get_vfo (rig, &tvfo);
+    if (RIG_OK != err) return err;
+  }
+
   switch (tvfo) {
   case RIG_VFO_A:
   case RIG_VFO_MAIN:
@@ -1108,9 +1125,9 @@ int kenwood_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
     rig_debug(RIG_DEBUG_ERR, "%s: unsupported VFO %d\n", __func__, vfo);
     return -RIG_EINVAL;
   }
-  sprintf(freqbuf, "F%c%011"PRIll, vfo_letter, (int64_t)freq);
+  snprintf(freqbuf, sizeof (freqbuf), "F%c%011"PRIll, vfo_letter, (int64_t)freq);
 
-  int err = kenwood_transaction(rig, freqbuf, NULL, 0);
+  err = kenwood_transaction(rig, freqbuf, NULL, 0);
 
   struct kenwood_priv_data * priv = rig->state.priv;
   if (RIG_OK == err && RIG_MODEL_TS590S == rig->caps->rig_model && priv->fw_rev_uint <= 107 && ('A' == vfo_letter || 'B' == vfo_letter))
@@ -1186,11 +1203,17 @@ int kenwood_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
   char freqbuf[50];
   char cmdbuf[4];
   int retval;
-  unsigned char vfo_letter;
+  unsigned char vfo_letter = '\0';
   vfo_t tvfo;
 
   tvfo = (vfo == RIG_VFO_CURR || vfo==RIG_VFO_VFO) ? rig->state.current_vfo : vfo;
 
+  if (RIG_VFO_CURR == tvfo) {
+    /* fetch from rig */
+    retval = rig_get_vfo (rig, &tvfo);
+    if (RIG_OK != retval) return retval;
+  }
+
   /* memory frequency cannot be read with an Fx command, use IF */
   if (tvfo == RIG_VFO_MEM) {
 
@@ -1210,12 +1233,11 @@ int kenwood_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
     vfo_letter = 'C';
     break;
   default:
-    rig_debug(RIG_DEBUG_ERR, "%s: unsupported VFO %d\n",
-        __func__, vfo);
+    rig_debug(RIG_DEBUG_ERR, "%s: unsupported VFO %d\n", __func__, vfo);
     return -RIG_EINVAL;
   }
 
-  sprintf(cmdbuf, "F%c", vfo_letter);
+  snprintf(cmdbuf, sizeof (cmdbuf), "F%c", vfo_letter);
 
   retval = kenwood_safe_transaction(rig, cmdbuf, freqbuf, 50, 13);
   if (retval != RIG_OK)
@@ -1265,13 +1287,13 @@ int kenwood_set_rit(RIG * rig, vfo_t vfo, shortfreq_t rit)
   if (rit == 0)
     return kenwood_transaction(rig, "RC", NULL, 0);
 
-  sprintf(buf, "R%c", (rit > 0) ? 'U' : 'D');
+  snprintf(buf, sizeof (buf), "R%c", (rit > 0) ? 'U' : 'D');
 
   retval = kenwood_transaction(rig, "RC", NULL, 0);
   if (retval != RIG_OK)
     return retval;
 
-  for (i = 0; i < abs(rint(rit/10)); i++)
+  for (i = 0; i < labs(lrint(rit/10)); i++)
     retval = kenwood_transaction(rig, buf, NULL, 0);
 
   return retval;
@@ -1361,6 +1383,7 @@ int kenwood_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
   if (!rig)
     return -RIG_EINVAL;
 
+  struct kenwood_priv_data *priv = rig->state.priv;
   struct kenwood_priv_caps *caps = kenwood_caps(rig);
   char buf[6];
   char kmode;
@@ -1392,6 +1415,14 @@ int kenwood_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
         }
     }
 
+  if (priv->is_emulation)
+    {
+      /* emulations like PowerSDR and SmartSDR normally hijack the
+         RTTY modes for SSB-DATA AFSK modes */
+      if (RIG_MODE_PKTLSB == mode) mode = RIG_MODE_RTTY;
+      if (RIG_MODE_PKTUSB == mode) mode = RIG_MODE_RTTYR;
+    }
+
   kmode = rmode2kenwood(mode, caps->mode_table);
   if (kmode < 0 ) {
     rig_debug(RIG_DEBUG_WARN, "%s: unsupported mode '%s'\n",
@@ -1424,7 +1455,7 @@ int kenwood_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
           err = kenwood_set_vfo_main_sub (rig, vfo);
           if (err != RIG_OK) return err;
         }
-      sprintf(buf, "OM0%c", c); /* target vfo is ignored */
+      snprintf(buf, sizeof (buf), "OM0%c", c); /* target vfo is ignored */
       int err = kenwood_transaction(rig, buf, NULL, 0);
       if (vfo != RIG_VFO_CURR && vfo != curr_vfo)
         {
@@ -1434,7 +1465,7 @@ int kenwood_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
     }
   else
     {
-      sprintf(buf, "MD%c", '0' + kmode);
+      snprintf(buf, sizeof (buf), "MD%c", '0' + kmode);
       err = kenwood_transaction(rig, buf, NULL, 0);
     }
   if (err != RIG_OK) return err;
@@ -1443,16 +1474,16 @@ int kenwood_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
       || RIG_MODEL_TS590SG == rig->caps->rig_model)
     {
       if (!(RIG_MODE_CW == mode
-         || RIG_MODE_CWR == mode
-         || RIG_MODE_AM == mode
-         || RIG_MODE_RTTY == mode
-	 || RIG_MODE_RTTYR == mode))
+            || RIG_MODE_CWR == mode
+            || RIG_MODE_AM == mode
+            || RIG_MODE_RTTY == mode
+            || RIG_MODE_RTTYR == mode))
         {
-	  /* supports DATA sub modes - see above */
-	  sprintf (buf, "DA%c", data_mode);
-	  err = kenwood_transaction (rig, buf, NULL, 0);
-	  if (err != RIG_OK) return err;
-	}
+          /* supports DATA sub modes - see above */
+          snprintf (buf, sizeof (buf), "DA%c", data_mode);
+          err = kenwood_transaction (rig, buf, NULL, 0);
+          if (err != RIG_OK) return err;
+        }
     }
 
   if (RIG_PASSBAND_NOCHANGE == width) return RIG_OK;
@@ -1526,6 +1557,7 @@ int kenwood_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
   if (!rig || !mode || !width)
     return -RIG_EINVAL;
 
+  struct kenwood_priv_data *priv = rig->state.priv;
   struct kenwood_priv_caps *caps = kenwood_caps(rig);
   char cmd[4];
   char modebuf[10];
@@ -1551,12 +1583,12 @@ int kenwood_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
           rig_debug(RIG_DEBUG_ERR, "%s: unsupported VFO %d\n", __func__, vfo);
           return -RIG_EINVAL;
         }
-      sprintf(cmd, "OM%c", c);
+      snprintf(cmd, sizeof (cmd), "OM%c", c);
       offs = 3;
     }
   else
     {
-      sprintf(cmd, "MD");
+      snprintf(cmd, sizeof (cmd), "MD");
       offs = 2;
     }
 
@@ -1574,6 +1606,13 @@ int kenwood_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
       kmode = modebuf[offs] - 'A' + 10;
     }
   *mode = kenwood2rmode(kmode, caps->mode_table);
+  if (priv->is_emulation)
+    {
+      /* emulations like PowerSDR and SmartSDR normally hijack the
+         RTTY modes for SSB-DATA AFSK modes */
+      if (RIG_MODE_RTTY == *mode) *mode = RIG_MODE_PKTLSB;
+      if (RIG_MODE_RTTYR == *mode) *mode = RIG_MODE_PKTUSB;
+    }
 
   if (RIG_MODEL_TS590S == rig->caps->rig_model
       || RIG_MODEL_TS590SG == rig->caps->rig_model)
@@ -1651,38 +1690,42 @@ int kenwood_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val)
 
   switch (level) {
   case RIG_LEVEL_RFPOWER:
-    /* XXX check level range */
-    sprintf(levelbuf, "PC%03d", kenwood_val);
+    /*
+     * Best estimate: 1.0 corresponds to 100W
+     * Anything better must be done in rig-specific files.
+     */
+    if (RIG_LEVEL_IS_FLOAT(level)) kenwood_val = val.f * 100;
+    snprintf(levelbuf, sizeof (levelbuf), "PC%03d", kenwood_val);
     break;
 
   case RIG_LEVEL_AF:
-    sprintf(levelbuf, "AG%03d", kenwood_val);
+    snprintf(levelbuf, sizeof (levelbuf), "AG%03d", kenwood_val);
     break;
 
   case RIG_LEVEL_RF:
     /* XXX check level range */
-    sprintf(levelbuf, "RG%03d", kenwood_val);
+    snprintf(levelbuf, sizeof (levelbuf), "RG%03d", kenwood_val);
     break;
 
   case RIG_LEVEL_SQL:
-    sprintf(levelbuf, "SQ%03d", kenwood_val);
+    snprintf(levelbuf, sizeof (levelbuf), "SQ%03d", kenwood_val);
     break;
 
   case RIG_LEVEL_AGC:
     if (kenwood_val > 3)
       kenwood_val = 3; /* 0.. 255 */
-    sprintf(levelbuf, "GT%03d", 84*kenwood_val);
+    snprintf(levelbuf, sizeof (levelbuf), "GT%03d", 84*kenwood_val);
     break;
 
   case RIG_LEVEL_ATT:
     /* set the attenuator if a correct value is entered */
     if (val.i == 0)
-      sprintf(levelbuf, "RA00");
+      snprintf(levelbuf, sizeof (levelbuf), "RA00");
     else {
       for (i=0; i<MAXDBLSTSIZ && rig->state.attenuator[i]; i++) {
         if (val.i == rig->state.attenuator[i])
         {
-          sprintf(levelbuf, "RA%02d", i+1);
+          snprintf(levelbuf, sizeof (levelbuf), "RA%02d", i+1);
           break;
         }
       }
@@ -1694,12 +1737,12 @@ int kenwood_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val)
   case RIG_LEVEL_PREAMP:
     /* set the preamp if a correct value is entered */
     if (val.i == 0)
-      sprintf(levelbuf, "PA0");
+      snprintf(levelbuf, sizeof (levelbuf), "PA0");
     else {
       for (i=0; i<MAXDBLSTSIZ && rig->state.preamp[i]; i++) {
         if (val.i == rig->state.preamp[i])
         {
-          sprintf(levelbuf, "PA%01d", i+1);
+          snprintf(levelbuf, sizeof (levelbuf), "PA%01d", i+1);
           break;
         }
       }
@@ -1711,25 +1754,25 @@ int kenwood_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val)
   case RIG_LEVEL_SLOPE_HIGH:
     if(val.i>20 || val.i < 0)
       return -RIG_EINVAL;
-    sprintf(levelbuf, "SH%02d",(val.i));
+    snprintf(levelbuf, sizeof (levelbuf), "SH%02d",(val.i));
     break;
 
   case RIG_LEVEL_SLOPE_LOW:
     if(val.i>20 || val.i < 0)
       return -RIG_EINVAL;
-    sprintf(levelbuf, "SL%02d",(val.i));
+    snprintf(levelbuf, sizeof (levelbuf), "SL%02d",(val.i));
     break;
 
   case RIG_LEVEL_CWPITCH:
     if(val.i > 1000 || val.i < 400)
       return -RIG_EINVAL;
-    sprintf(levelbuf, "PT%02d", (val.i / 50) - 8);
+    snprintf(levelbuf, sizeof (levelbuf), "PT%02d", (val.i / 50) - 8);
     break;
 
   case RIG_LEVEL_KEYSPD:
     if(val.i > 50 || val.i < 5)
       return -RIG_EINVAL;
-    sprintf(levelbuf, "KS%03d", val.i);
+    snprintf(levelbuf, sizeof (levelbuf), "KS%03d", val.i);
     break;
 
   default:
@@ -1853,7 +1896,14 @@ int kenwood_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
     break;
 
   case RIG_LEVEL_RFPOWER:
-    return get_kenwood_level(rig, "PC", &val->f);
+    /*
+     * an answer "PC100" means 100 Watt
+     * which is val=1.0 on most rigs, but
+     * get_kenwood_level maps 0...255 onto 0.0 ... 1.0
+     */
+    ret = get_kenwood_level(rig, "PC", &val->f);
+    val->f = val->f * (255.0/100.0);
+    return ret;
 
   case RIG_LEVEL_AF:
     return get_kenwood_level(rig, "AG", &val->f);
@@ -1937,59 +1987,59 @@ int kenwood_set_func(RIG *rig, vfo_t vfo, setting_t func, int status)
 
   switch (func) {
   case RIG_FUNC_NB:
-    sprintf(buf, "NB%c", (status == 0) ? '0' : '1');
+    snprintf(buf, sizeof (buf), "NB%c", (status == 0) ? '0' : '1');
     return kenwood_transaction(rig, buf, NULL, 0);
 
   case RIG_FUNC_ABM:
-    sprintf(buf, "AM%c", (status == 0) ? '0' : '1');
+    snprintf(buf, sizeof (buf), "AM%c", (status == 0) ? '0' : '1');
     return kenwood_transaction(rig, buf, NULL, 0);
 
   case RIG_FUNC_COMP:
-    sprintf(buf, "PR%c", (status == 0) ? '0' : '1');
+    snprintf(buf, sizeof (buf), "PR%c", (status == 0) ? '0' : '1');
     return kenwood_transaction(rig, buf, NULL, 0);
 
   case RIG_FUNC_TONE:
-    sprintf(buf, "TO%c", (status == 0) ? '0' : '1');
+    snprintf(buf, sizeof (buf), "TO%c", (status == 0) ? '0' : '1');
     return kenwood_transaction(rig, buf, NULL, 0);
 
   case RIG_FUNC_TSQL:
-    sprintf(buf, "CT%c", (status == 0) ? '0' : '1');
+    snprintf(buf, sizeof (buf), "CT%c", (status == 0) ? '0' : '1');
     return kenwood_transaction(rig, buf, NULL, 0);
 
   case RIG_FUNC_VOX:
-    sprintf(buf, "VX%c", (status == 0) ? '0' : '1');
+    snprintf(buf, sizeof (buf), "VX%c", (status == 0) ? '0' : '1');
     return kenwood_transaction(rig, buf, NULL, 0);
 
   case RIG_FUNC_FAGC:
-    sprintf(buf, "GT00%c", (status == 0) ? '4' : '2');
+    snprintf(buf, sizeof (buf), "GT00%c", (status == 0) ? '4' : '2');
     return kenwood_transaction(rig, buf, NULL, 0);
 
   case RIG_FUNC_NR:
-    sprintf(buf, "NR%c", (status == 0) ? '0' : '1');
+    snprintf(buf, sizeof (buf), "NR%c", (status == 0) ? '0' : '1');
     return kenwood_transaction(rig, buf, NULL, 0);
 
   case RIG_FUNC_BC:
-    sprintf(buf, "BC%c", (status == 0) ? '0' : '1');
+    snprintf(buf, sizeof (buf), "BC%c", (status == 0) ? '0' : '1');
     return kenwood_transaction(rig, buf, NULL, 0);
 
   case RIG_FUNC_ANF:
-    sprintf(buf, "NT%c", (status == 0) ? '0' : '1');
+    snprintf(buf, sizeof (buf), "NT%c", (status == 0) ? '0' : '1');
     return kenwood_transaction(rig, buf, NULL, 0);
 
   case RIG_FUNC_LOCK:
-    sprintf(buf, "LK%c", (status == 0) ? '0' : '1');
+    snprintf(buf, sizeof (buf), "LK%c", (status == 0) ? '0' : '1');
     return kenwood_transaction(rig, buf, NULL, 0);
 
   case RIG_FUNC_AIP:
-    sprintf(buf, "MX%c", (status == 0) ? '0' : '1');
+    snprintf(buf, sizeof (buf), "MX%c", (status == 0) ? '0' : '1');
     return kenwood_transaction(rig, buf, NULL, 0);
 
   case RIG_FUNC_RIT:
-    sprintf(buf, "RT%c", (status == 0) ? '0' : '1');
+    snprintf(buf, sizeof (buf), "RT%c", (status == 0) ? '0' : '1');
     return kenwood_transaction(rig, buf, NULL, 0);
 
   case RIG_FUNC_XIT:
-    sprintf(buf, "XT%c", (status == 0) ? '0' : '1');
+    snprintf(buf, sizeof (buf), "XT%c", (status == 0) ? '0' : '1');
     return kenwood_transaction(rig, buf, NULL, 0);
 
   default:
@@ -2116,7 +2166,7 @@ int kenwood_set_ctcss_tone(RIG *rig, vfo_t vfo, tone_t tone)
     return -RIG_EINVAL;
 
 /* TODO: replace menu no 57 by a define */
-  sprintf(tonebuf, "EX%03d%04d", 57, i+1);
+  snprintf(tonebuf, sizeof (tonebuf), "EX%03d%04d", 57, i+1);
 
   return kenwood_transaction(rig, tonebuf, NULL, 0);
 }
@@ -2161,11 +2211,11 @@ int kenwood_set_ctcss_tone_tn(RIG *rig, vfo_t vfo, tone_t tone)
           rig_debug(RIG_DEBUG_ERR, "%s: unsupported VFO %d\n", __func__, vfo);
           return -RIG_EINVAL;
         }
-      sprintf(buf, "TN%c%02d", c, i + 1);
+      snprintf(buf, sizeof (buf), "TN%c%02d", c, i + 1);
     }
   else
     {
-      sprintf(buf, "TN%02d", i + 1);
+      snprintf(buf, sizeof (buf), "TN%02d", i + 1);
     }
 
   return kenwood_transaction(rig, buf, NULL, 0);
@@ -2211,7 +2261,7 @@ int kenwood_get_ctcss_tone(RIG *rig, vfo_t vfo, tone_t *tone)
           rig_debug(RIG_DEBUG_ERR, "%s: unsupported VFO %d\n", __func__, vfo);
           return -RIG_EINVAL;
         }
-      sprintf(cmd, "TN%c", c);
+      snprintf(cmd, sizeof (cmd), "TN%c", c);
       retval = kenwood_safe_transaction (rig, cmd, buf, sizeof (buf), 5);
       memcpy (tonebuf, &buf[3], 2);
     }
@@ -2285,11 +2335,11 @@ int kenwood_set_ctcss_sql(RIG *rig, vfo_t vfo, tone_t tone)
           rig_debug(RIG_DEBUG_ERR, "%s: unsupported VFO %d\n", __func__, vfo);
           return -RIG_EINVAL;
         }
-      sprintf(buf, "CN%c%02d", c, i + 1);
+      snprintf(buf, sizeof (buf), "CN%c%02d", c, i + 1);
     }
   else
     {
-      sprintf(buf, "CN%02d", i + 1);
+      snprintf(buf, sizeof (buf), "CN%02d", i + 1);
     }
   return kenwood_transaction(rig, buf, NULL, 0);
 }
@@ -2302,7 +2352,7 @@ int kenwood_get_ctcss_sql(RIG *rig, vfo_t vfo, tone_t *tone)
     return -RIG_EINVAL;
 
   const struct rig_caps *caps;
-  char cmd[3];
+  char cmd[4];
   char tonebuf[6];
   int offs;
   int i, retval;
@@ -2329,12 +2379,12 @@ int kenwood_get_ctcss_sql(RIG *rig, vfo_t vfo, tone_t *tone)
           rig_debug(RIG_DEBUG_ERR, "%s: unsupported VFO %d\n", __func__, vfo);
           return -RIG_EINVAL;
         }
-      sprintf(cmd, "CN%c", c);
+      snprintf(cmd, sizeof (cmd), "CN%c", c);
       offs = 3;
     }
   else
     {
-      sprintf(cmd, "CT");
+      snprintf(cmd, sizeof (cmd), "CT");
       offs = 2;
     }
 
@@ -2406,11 +2456,11 @@ int kenwood_set_ant(RIG * rig, vfo_t vfo, ant_t ant)
           rig_debug(RIG_DEBUG_ERR, "%s: unsupported VFO %d\n", __func__, vfo);
           return -RIG_EINVAL;
         }
-      sprintf (cmd, "AN0%c%c99", c, a);
+      snprintf (cmd, sizeof (cmd), "AN0%c%c99", c, a);
     }
   else
     {
-      sprintf (cmd, "AN%c", a);
+      snprintf (cmd, sizeof (cmd), "AN%c", a);
     }
 
   return kenwood_transaction(rig, cmd, NULL, 0);
@@ -2696,7 +2746,7 @@ int kenwood_reset(RIG *rig, reset_t reset)
         return -RIG_EINVAL;
       }
   }
-  sprintf(rstbuf, "SR%c", rst);
+  snprintf(rstbuf, sizeof (rstbuf), "SR%c", rst);
 
   /* this command has no answer */
   return kenwood_transaction(rig, rstbuf, NULL, 0);
@@ -2713,7 +2763,7 @@ int kenwood_send_morse(RIG *rig, vfo_t vfo, const char *msg)
     return -RIG_EINVAL;
 
   char morsebuf[30], m2[30];
-  int msg_len, buff_len, retval;
+  int msg_len, buff_len, retval, i;
   const char *p;
 
   p = msg;
@@ -2721,20 +2771,43 @@ int kenwood_send_morse(RIG *rig, vfo_t vfo, const char *msg)
 
   while(msg_len > 0) {
     /*
-     * TODO: check with "KY" if char buffer is available.
-     *    if not, sleep.
-     *
-     * Make the total message segments 28 characters
-     * in length because Kenwood demands it.
-     * Spaces fill in the message end.
+     * Check with "KY" if char buffer is available.
+     * if not, sleep.
      */
+     for (;;) {
+       retval = kenwood_transaction(rig, "KY;", m2, 4);
+       if (retval != RIG_OK)
+         return retval;
+       /*
+        * If answer is "KY0;", there is space in buffer and we can proceed.
+        * If answer is "KY1;", we have to wait a while
+        * If answer is something else, return with error to prevent infinite loops
+        */
+       if (!strncmp(m2,"KY0", 3)) break;
+       if (!strncmp(m2,"KY1", 3)) usleep(500000); else return -RIG_EINVAL;
+    }
     buff_len = msg_len > 24 ? 24 : msg_len;
 
     strncpy(m2, p, 24);
     m2[24] = '\0';
 
-    /* the command must consist of 28 bytes */
-    sprintf(morsebuf, "KY %-24s", m2);
+    /*
+     * Make the total message segments 28 characters
+     * in length because some Kenwoods demand it.
+     * 0x20 fills in the message end.
+     * Some rigs don't need the fill
+     */
+    switch(rig->caps->rig_model) {
+    case RIG_MODEL_K3: // probably a lot more rigs need to go here
+      snprintf(morsebuf, sizeof (morsebuf), "KY %s", m2);
+      break;
+    default:
+      /* the command must consist of 28 bytes 0x20 padded */
+      snprintf(morsebuf, sizeof (morsebuf), "KY %-24s", m2);
+      for(i = strlen(morsebuf) - 1;i > 0 && morsebuf[i] == ' '; --i) {
+        morsebuf[i]=0x20;
+      }
+    }
     retval = kenwood_transaction(rig, morsebuf, NULL, 0);
     if (retval != RIG_OK)
       return retval;
@@ -2808,7 +2881,7 @@ int kenwood_set_mem(RIG *rig, vfo_t vfo, int ch)
           rig_debug(RIG_DEBUG_ERR, "%s: unsupported VFO %d\n", __func__, vfo);
           return -RIG_EINVAL;
         }
-      sprintf (buf, "MN%c%03d", c, ch);
+      snprintf (buf, sizeof (buf), "MN%c%03d", c, ch);
     }
   else
     {
@@ -2817,7 +2890,7 @@ int kenwood_set_mem(RIG *rig, vfo_t vfo, int ch)
        * where b is the bank number, mm the memory number.
        * b can be a space
        */
-      sprintf(buf, "MC %02d", ch);
+      snprintf(buf, sizeof (buf), "MC %02d", ch);
     }
   return kenwood_transaction(rig, buf, NULL, 0);
 }
@@ -2856,7 +2929,7 @@ int kenwood_get_mem(RIG *rig, vfo_t vfo, int *ch)
           rig_debug(RIG_DEBUG_ERR, "%s: unsupported VFO %d\n", __func__, vfo);
           return -RIG_EINVAL;
         }
-      sprintf (cmd, "MN%c", c);
+      snprintf (cmd, sizeof (cmd), "MN%c", c);
       offs = 3;
     }
   else
@@ -2866,7 +2939,7 @@ int kenwood_get_mem(RIG *rig, vfo_t vfo, int *ch)
        * where b is the bank number, mm the memory number.
        * b can be a space
        */
-      sprintf (cmd, "MC");
+      snprintf (cmd, sizeof (cmd), "MC");
       offs = 2;
     }
   retval = kenwood_safe_transaction(rig, cmd, membuf, sizeof (membuf), 3 + offs);
@@ -2919,7 +2992,7 @@ int kenwood_get_channel(RIG *rig, channel_t *chan)
     {
       bank = '0' + chan->bank_num;
     }
-  sprintf(cmd, "MR0%c%02d", bank, chan->channel_num);
+  snprintf(cmd, sizeof (cmd), "MR0%c%02d", bank, chan->channel_num);
 
   err = kenwood_safe_transaction(rig, cmd, buf, 26, 23);
   if (err != RIG_OK)
@@ -3038,7 +3111,7 @@ int kenwood_set_channel(RIG *rig, const channel_t *chan)
       bank = '0' + chan->bank_num;
     }
 
-  sprintf(buf, "MW0%c%02d%011"PRIll"%c%c%c%02d ", /* note the space at
+  snprintf(buf, sizeof (buf), "MW0%c%02d%011"PRIll"%c%c%c%02d ", /* note the space at
                                                      the end */
           bank,
           chan->channel_num,
@@ -3052,7 +3125,7 @@ int kenwood_set_channel(RIG *rig, const channel_t *chan)
   if (err != RIG_OK)
     return err;
 
-  sprintf(buf, "MW1%c%02d%011"PRIll"%c%c%c%02d ",
+  snprintf(buf, sizeof (buf), "MW1%c%02d%011"PRIll"%c%c%c%02d ",
           bank,
           chan->channel_num,
           (int64_t)(chan->split == RIG_SPLIT_ON ? chan->tx_freq : 0),
@@ -3078,15 +3151,15 @@ int kenwood_set_ext_parm(RIG *rig, token_t token, value_t val)
     return kenwood_transaction(rig, "VR", NULL, 0);
 
   case TOK_FINE:
-    sprintf(buf, "FS%c", (val.i == 0) ? '0' : '1');
+    snprintf(buf, sizeof (buf), "FS%c", (val.i == 0) ? '0' : '1');
     return kenwood_transaction(rig, buf, NULL, 0);
 
   case TOK_XIT:
-    sprintf(buf, "XT%c", (val.i == 0) ? '0' : '1');
+    snprintf(buf, sizeof (buf), "XT%c", (val.i == 0) ? '0' : '1');
     return kenwood_transaction(rig, buf, NULL, 0);
 
   case TOK_RIT:
-    sprintf(buf, "RT%c", (val.i == 0) ? '0' : '1');
+    snprintf(buf, sizeof (buf), "RT%c", (val.i == 0) ? '0' : '1');
     return kenwood_transaction(rig, buf, NULL, 0);
   }
 
@@ -3285,47 +3358,48 @@ DECLARE_INITRIG_BACKEND(kenwood)
 
 //  rig_debug(RIG_DEBUG_VERBOSE, "kenwood: _init called\n");
 
-	rig_register(&ts950sdx_caps);
-	rig_register(&ts50s_caps);
-	rig_register(&ts140_caps);
-	rig_register(&ts450s_caps);
-	rig_register(&ts570d_caps);
-	rig_register(&ts570s_caps);
-	rig_register(&ts680s_caps);
-	rig_register(&ts690s_caps);
-	rig_register(&ts790_caps);
-	rig_register(&ts850_caps);
-	rig_register(&ts870s_caps);
-	rig_register(&ts930_caps);
-	rig_register(&ts2000_caps);
-	rig_register(&trc80_caps);
-	rig_register(&k2_caps);
-	rig_register(&k3_caps);
-	rig_register(&xg3_caps);
+  rig_register(&ts950sdx_caps);
+  rig_register(&ts50s_caps);
+  rig_register(&ts140_caps);
+  rig_register(&ts450s_caps);
+  rig_register(&ts570d_caps);
+  rig_register(&ts570s_caps);
+  rig_register(&ts680s_caps);
+  rig_register(&ts690s_caps);
+  rig_register(&ts790_caps);
+  rig_register(&ts850_caps);
+  rig_register(&ts870s_caps);
+  rig_register(&ts930_caps);
+  rig_register(&ts2000_caps);
+  rig_register(&trc80_caps);
+  rig_register(&k2_caps);
+  rig_register(&k3_caps);
+  rig_register(&xg3_caps);
 
-	rig_register(&ts440_caps);
-	rig_register(&ts940_caps);
-	rig_register(&ts711_caps);
-	rig_register(&ts811_caps);
-	rig_register(&r5000_caps);
+  rig_register(&ts440_caps);
+  rig_register(&ts940_caps);
+  rig_register(&ts711_caps);
+  rig_register(&ts811_caps);
+  rig_register(&r5000_caps);
 
-	rig_register(&tmd700_caps);
-	rig_register(&thd7a_caps);
-	rig_register(&thd72a_caps);
-	rig_register(&thf7e_caps);
-	rig_register(&thg71_caps);
-	rig_register(&tmv7_caps);
-	rig_register(&tmd710_caps);
+  rig_register(&tmd700_caps);
+  rig_register(&thd7a_caps);
+  rig_register(&thd72a_caps);
+  rig_register(&thf7e_caps);
+  rig_register(&thg71_caps);
+  rig_register(&tmv7_caps);
+  rig_register(&tmd710_caps);
 
-	rig_register(&ts590_caps);
-	rig_register(&ts990s_caps);
-	rig_register(&ts590sg_caps);
-	rig_register(&ts480_caps);
-	rig_register(&thf6a_caps);
+  rig_register(&ts590_caps);
+  rig_register(&ts990s_caps);
+  rig_register(&ts590sg_caps);
+  rig_register(&ts480_caps);
+  rig_register(&thf6a_caps);
 
   rig_register(&transfox_caps);
 
   rig_register(&f6k_caps);
+  rig_register(&pihpsdr_caps);
 
   return RIG_OK;
 }
