@@ -33,7 +33,7 @@
 #define EOM_TH '\r'
 
 #define KENWOOD_MODE_TABLE_MAX  24
-#define KENWOOD_MAX_BUF_LEN   50 /* max answer len, arbitrary */
+#define KENWOOD_MAX_BUF_LEN   128 /* max answer len, arbitrary */
 
 
 /* Tokens for Parameters common to multiple rigs.
@@ -78,6 +78,7 @@ struct kenwood_priv_data {
     int trn_state;  /* AI state discovered at startup */
     unsigned fw_rev_uint; /* firmware revison as a number 1.07 -> 107 */
     char verify_cmd[4];   /* command used to verify set commands */
+    int is_emulation;     /* flag for TS-2000 emulations */
     void * data;          /* model specific data */
 };
 
@@ -200,6 +201,7 @@ extern const struct rig_caps thf6a_caps;
 extern const struct rig_caps transfox_caps;
 
 extern const struct rig_caps f6k_caps;
+extern const struct rig_caps pihpsdr_caps;
 
 /* use when not interested in the answer, but want to check its len */
 static int inline kenwood_simple_transaction(RIG *rig, const char *cmd, size_t expected)
