@@ -184,7 +184,7 @@ th_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
 
 	freq5=round(freq/5000)*5000;
 	freq625=round(freq/6250)*6250;
-	if (abs(freq5-freq)<abs(freq625-freq)) {
+	if (fabs(freq5-freq)<fabs(freq625-freq)) {
 	  step=0;
 	  freq_sent=freq5;
 	}
@@ -374,9 +374,9 @@ th_set_vfo(RIG *rig, vfo_t vfo)
 				return retval;
 	}
 
-    /* No "VMC" cmd on THD72A */
-    if (rig->caps->rig_model == RIG_MODEL_THD72A)
-        return RIG_OK;
+	/* No "VMC" cmd on THD72A */
+	if (rig->caps->rig_model == RIG_MODEL_THD72A)
+        	return RIG_OK;
 
 	/* set vfo */
 	switch (vfo) {
@@ -980,14 +980,14 @@ th_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 		if (retval != RIG_OK)
 			return retval;
 
-			retval = sscanf(ackbuf, "AG %d,%x", &v, &l);
-			if (retval != 2 || l < rig->caps->level_gran[LVL_AF].min.i || l > rig->caps->level_gran[LVL_AF].max.i) {
-				rig_debug(RIG_DEBUG_ERR, "%s: Unexpected reply '%s'\n", __func__, ackbuf);
-				return -RIG_ERJCTED;
-			}
+		retval = sscanf(ackbuf, "AG %d,%x", &v, &l);
+		if (retval != 2 || l < rig->caps->level_gran[LVL_AF].min.i || l > rig->caps->level_gran[LVL_AF].max.i) {
+			rig_debug(RIG_DEBUG_ERR, "%s: Unexpected reply '%s'\n", __func__, ackbuf);
+			return -RIG_ERJCTED;
+		}
 
-			/* range [0.0 ... 1.0] */
-			val->f = (float)(l-rig->caps->level_gran[LVL_AF].min.i) /
+		/* range [0.0 ... 1.0] */
+		val->f = (float)(l-rig->caps->level_gran[LVL_AF].min.i) /
 		(float)(rig->caps->level_gran[LVL_AF].max.i-rig->caps->level_gran[LVL_AF].min.i);
 		break;
 
@@ -1941,7 +1941,7 @@ int th_set_channel(RIG *rig, const channel_t *chan)
 		retval = sprintf(membuf, "%s,%011"PRIll",%X,%d,%d,%d,%d,%d,%02d,%02d,%03d,%09"PRIll",%d%s",
 			req, (int64_t)chan->freq, step, shift, rev, tone,
 			ctcss, dcs, tonefq, ctcssfq, dcscode,
-			(int64_t)abs(chan->rptr_offs), mode, lockoutstr
+			(int64_t)labs((long)(chan->rptr_offs)), mode, lockoutstr
 			);
 	} else {
 
@@ -1949,7 +1949,7 @@ int th_set_channel(RIG *rig, const channel_t *chan)
 		retval = sprintf(membuf, "%s,%011"PRIll",%X,%d,%d,%d,%d,,%02d,,%02d,%09"PRIll"%s",
 			req, (int64_t)chan->freq, step, shift, rev, tone,
 			ctcss, tonefq, ctcssfq,
-			(int64_t)abs(chan->rptr_offs), lockoutstr
+			(int64_t)labs((long)(chan->rptr_offs)), lockoutstr
 			);
 	}
 
