@@ -1,21 +1,22 @@
 /*
  *  Hamlib Tentec backend - main header
- *  Copyright (c) 2001-2010 by Stephane Fillod
+ *  Copyright (c) 2001-2011 by Stephane Fillod
  *
  *
- *   This library is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU Library General Public License as
- *   published by the Free Software Foundation; either version 2 of
- *   the License, or (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License as published by the Free Software Foundation; either
+ *   version 2.1 of the License, or (at your option) any later version.
  *
- *   This program is distributed in the hope that it will be useful,
+ *   This library is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU Library General Public License for more details.
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *   Lesser General Public License for more details.
  *
- *   You should have received a copy of the GNU Library General Public
+ *   You should have received a copy of the GNU Lesser General Public
  *   License along with this library; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ *
  */
 
 /**
@@ -40,7 +41,7 @@ struct tentec_priv_data {
 	freq_t freq;		/*!< tuned frequency */
 	pbwidth_t width;	/*!< filter bandwidth in Hz */
 	int cwbfo;		    /*!< BFO frequency: 1000 [0-2000Hz] */
-    int pbt;            /*!< Passband Tuning, IF shift: 0 [-2000Hz to 2000Hz] */ 
+    int pbt;            /*!< Passband Tuning, IF shift: 0 [-2000Hz to 2000Hz] */
 	float lnvol;		/*!< line-out volume: 30 [0..63] */
 	float spkvol;		/*!< speaker volume: 30 [0..63] */
 	int agc;		/*!< AGC: medium */
@@ -73,6 +74,7 @@ extern const struct rig_caps tt585_caps;
 extern const struct rig_caps tt588_caps;
 extern const struct rig_caps tt550_caps;
 extern const struct rig_caps tt565_caps;
+extern const struct rig_caps tt599_caps;
 extern const struct rig_caps rx331_caps;
 
 
